(* File lexer.mll *)

{
open Lexing
open Parser

exception Lexical_error of int * int
}

rule token = parse
    [' ''\t''\n'] 	{ token lexbuf }
  | ['0'-'9']+      	{ INT(int_of_string (lexeme lexbuf)) }
  | "-" 	    	{ MINUS }
  | "="			{ EQ }
  | "!="                { INFIX0 "!=" }
  | "<"                 { INFIX0 "<" }
  | ">"                 { INFIX0 ">" }
  | "<="                { INFIX0 "<=" }
  | ">="                { INFIX0 ">=" }
  | "+" 	    	{ INFIX1 "+" }
  | ['*''/']		{ INFIX2 (lexeme lexbuf) }
  | "->"		{ MINUSGREATER }
  | "if"		{ IF }
  | "then"		{ THEN }
  | "else"		{ ELSE }
  | "fun"		{ FUN }
  | "rec"		{ REC }
  | "let"		{ LET }
  | "def"		{ DEF }
  | ";;"		{ PTPT }
  | "in"		{ IN }
  | "("			{ LPAREN }
  | ")"			{ RPAREN }
  | ","                 { COMMA }
  | "true"              { BOOL(true) }
  | "false"             { BOOL(false) }
  | ['a'-'z''A'-'Z']
    ['a'-'z''A'-'Z''0'-'9']*
      	       	       	{ IDENT(lexeme lexbuf) }
  | eof			{ EOF }
  | _ 			{ raise (Lexical_error(lexeme_start lexbuf,
					       lexeme_end lexbuf)) }

