(* eliminer les variables libres *)
open Syntaxe_abstraite
open Vars
open Cmm

(* fonction auxiliaire *)
let symbol = ref 0
let new_symb () =
  symbol := !symbol + 1;
  "_" ^ (string_of_int !symbol)

let rec names l e =
  match e with
    Eint _ | Ebool _ -> e
  | Elocal(n) -> if List.mem n l then Elocal(n) else Eglobal(n)
  | Eglobal _ -> e
  | Eop(op, e1, e2) -> 
      Eop(op, names l e1, names l e2)
  | Eapp(e1, e2) ->
      Eapp(names l e1, names l e2)
  | Eifthenelse(e1, e2, e3) ->
      Eifthenelse(names l e1, names l e2, names l e3)
  | Efun(x, e1) ->
      Efun(x, names (x :: l) e1)
  | Elet(x, e1, e2) ->
      let e1 = names l e1 in
      Elet(x, e1, names (x :: l) e2)

let names d =
  match d with
    Edef(x, e) -> Edef(x, names [] e)

let names d_list = List.map names d_list
  
(* desimbrication et fermeture *)
let rec clos context e =
  match e with
    Eint(i) -> context, Mint(i)
  | Ebool(i) -> context, Mbool(i)
  | Elocal(n) -> context, Mlocal(n)
  | Eglobal(n) -> context, Mglobal(n)
  | Eop(op, e1, e2) -> 
      let context, e1 = clos context e1 in
      let context, e2 = clos context e2 in
      context, Mop(op, e1, e2)
  | Eapp(e1, e2) ->
      let context, e1 = clos context e1 in
      let context, e2 = clos context e2 in
      context, Mcall(e1,e2)
  | Eifthenelse(e1, e2, e3) ->
      let context, e1 = clos context e1 in
      let context, e2 = clos context e2 in
      let context, e3 = clos context e3 in
      context, Mifthenelse(e1, e2, e3)
  | Efun(x, e1) ->
      let v = vars e in
      let context, e1 = clos context e1 in
      let n = new_symb () in
      (Mfundef(n, v, x, e1)) :: context, Mclos(n, v)
  | Elet(x, e1, e2) ->
      let context, e1 = clos context e1 in
      let context, e2 = clos context e2 in
      context, Mlet(x, e1, e2)

let clos context d =
  match d with
    Edef(x, e) ->
      let context, c = clos context e in
      (Mdef(x, c)) :: context

let clos d_list =
  (* deux parcours inverses ici *)
  let d_list = List.fold_left clos [] d_list in
  List.rev d_list
  




