%{

open Syntaxe_abstraite

let make_expr desc = desc
let make_def desc = desc

let app f e = make_expr (Eapp(f,e))
let rec app_n f l =
  match l with
    [] -> f
  | x::l -> app_n (app f x) l

let binop_of_name s =
  match s with
    "=" -> Eegal
  | "!=" -> Enegal
  | "<" -> Einf
  | ">" -> Esup
  | "<=" -> Einf_egal
  | ">=" -> Esup_egal
  | "+" -> Eplus
  | "*" -> Emult
  | "-" -> Emoins
  | "/" -> Ediv
  | _ -> failwith "binop_of_name"

let infixop op e1 e2 = 
  make_expr (Eop(binop_of_name op,e1,e2))

%}

/* Tokens */
%token EOF
%token <string> IDENT
%token <int> INT
%token <bool> BOOL
%token <string> INFIX0  
%token <string> INFIX1
%token <string> INFIX2
%token DEF	/* "def" */
%token LPAREN  	/* "(" */
%token RPAREN  	/* ")" */
%token EQ   	/* "=" */
%token MINUS   	/* "-" */
%token MINUSGREATER /* "->" */
%token COMMA    /* "," */
%token IF       /* "if" */
%token THEN   	/* "then" */
%token ELSE   	/* "else" */
%token FUN	/* "fun" */
%token REC	/* "rec" */
%token LET    	/* "let" */
%token PTPT    	/* ";;" */
%token IN    	/* "in" */
%token EOF      /* "eof" */

/* precedences */

%right MINUSGREATER
%right prec_let
%right prec_if
%left COMMA
%left EQ
%left INFIX0
%right MINUS
%left INFIX1
%left INFIX2
%right prec_unary_minus
%left prec_app

/* points d'entre'e */
%start declaration_list
%type <Syntaxe_abstraite.def list> declaration_list
%%

declaration_list:
  declaration declaration_list
      { $1 :: $2 }
  | EOF
      { [] }
;

declaration : 
  DEF IDENT EQ expression PTPT
      { make_def (Edef($2,$4)) }
;

simple_expression :
  IDENT
      { make_expr (Elocal $1) }
  | INT
      { make_expr (Eint($1)) }
  | BOOL
      { make_expr (Ebool($1)) }
  | LPAREN expression RPAREN
      { $2 }
;

expression :
    simple_expression
      { $1 }
  | simple_expression simple_expression_list %prec prec_app
      { app_n $1 $2 }
  | LET IDENT EQ expression IN expression %prec prec_let
      { make_expr (Elet($2,$4,$6)) }
  | IF expression THEN expression ELSE expression %prec prec_if
      { make_expr (Eifthenelse($2,$4,$6)) }
  | expression EQ expression
      { infixop "=" $1 $3 }
  | expression MINUS expression
      { infixop "-" $1 $3 }
  | expression INFIX0 expression
      { infixop $2 $1 $3 }
  | expression INFIX1 expression
      { infixop $2 $1 $3 }
  | expression INFIX2 expression
      { infixop $2 $1 $3 }
  | FUN IDENT MINUSGREATER expression
      { make_expr (Efun($2,$4)) }
  | REC IDENT IDENT EQ expression
      { make_expr (Erec($2,$3,$5)) }
;

simple_expression_list :
    simple_expression simple_expression_list %prec prec_app
      { $1::$2 }
   | simple_expression
      { [$1] }
;

